from django.contrib import admin
from activity.models import Process
from activity.models import Activity


@admin.register(Process)
class ProcessAdmin(admin.ModelAdmin):
    list_display = ['id', 'process_key', 'process_type_slug', 'internal_key', 'status']

    search_fields = [
        'process_key',
        'internal_key',
    ]


@admin.register(Activity)
class ActivityAdmin(admin.ModelAdmin):
    list_display = ['activity_id', 'process', 'from_template_activity_slug']
    raw_id_fields = ['process']

    search_fields = [
        'activity_id',
        'process__process_key',
        'process__internal_key',
        'from_template_activity_slug',
    ]



from activity.models import Process
from activity.models import Activity


def _save_process(process_key, internal_key, process_type, status):
    process, created = Process.objects.update_or_create(
        internal_key=internal_key,
        process_type_slug=process_type,
        defaults={'status': status, 'process_key': process_key},
    )
    return process


def _save_activity(process_key, activity_id, from_template_activity_slug):
    process = Process.objects.get(process_key=process_key)

    activity, created = Activity.objects.update_or_create(
        activity_id=activity_id,
        process=process,
        from_template_activity_slug=from_template_activity_slug,
    )
    return activity


def _delete_process(process_key):
    process = Process.objects.get(process_key=process_key)
    process.delete()
    return process


def _delete_activity(activity_id):
    activity = Activity.objects.get(activity_id=activity_id)
    activity.delete()
    return activity


class BaseConsumerService(object):
    """
    Ronan queue consumer service.
    Ronan consumer will be notified if any one of these happened on ronan:
    - A process is created
    - A process is updated
    - A process is deleted
    - An activity is created
    - An activity is updated
    - An activity is deleted
    - An activity is completed
    - A submission of an activity is submitted
    - A submission of an activity is unsubmitted
    The webhook will be hit if the activity is under a process which has process_type of this service as consumer
    Depending on the event, consumer service may return some value on the request to be processed by ronan.
    """

    def on_process_created(self, process_data):
        """
        Invoked when a process is created.
        You may override this method to react on a process created.
        You may return these data as dict, so Ronan will react to them:
        - TBD
        :param process_data: ronan process that has been created
        :return: dictionary as specified on description
        """
        _save_process(
            process_data.get('process_key'),
            process_data.get('external_key'),
            process_data.get('process_type'),
            process_data.get('status'),
        )
        return {}

    def on_process_updated(self, process_data):
        """
        Invoked when a process is updated.
        You may override this method to react on a process updated.
        You may return these data as dict, so Ronan will react to them:
        - TBD
        :param process_data: ronan process that has been updated
        :return: dictionary as specified on description
        """
        _save_process(
            process_data.get('process_key'),
            process_data.get('external_key'),
            process_data.get('process_type'),
            process_data.get('status'),
        )
        return {}

    def on_process_deleted(self, process_data):
        """
        Invoked when a process is deleted.
        You may override this method to react on a process deleted.
        :param process_data: ronan process that has been deleted
        :return: empty dictionary
        """
        _delete_process(
            process_data.get('process_key'),
        )
        return {}

    def on_activity_created(self, activity_data):
        """
        Invoked when an activity is created.
        You may override this method to react on an activity created.
        You may return these data as dict, so Ronan will react to them:
        - responsible_user_ids: responsible user id that will be added
        - accountable_user_id: accountable user id
        :param activity_data: ronan activity that has been created
        :return: dictionary as specified on description
        """
        from_template_activity = activity_data.get('from_template_activity')
        from_template_activity_slug = None

        if from_template_activity is not None:
            from_template_activity_slug = from_template_activity.get('slug')

        _save_activity(
            activity_data.get('process').get('process_key'),
            activity_data.get('id'),
            from_template_activity_slug,
        )
        return {}

    def on_activity_updated(self, activity_data):
        """
        Invoked when an activity is updated.
        You may override this method to react on an activity updated.
        :param activity_data: ronan activity that has been updated
        :return: empty dictionary
        """
        return {}

    def on_activity_completed(self, activity_data):
        """
        Invoked when an activity is completed.
        You may override this method to react on an activity completed.
        :param activity_data: ronan activity that has been completed
        :return: empty dictionary
        """
        return {}

    def on_activity_deleted(self, activity_data):
        """
        Invoked when an activity is deleted.
        You may override this method to react on an activity deleted.
        :param activity_data: ronan activity that has been deleted
        :return: empty dictionary
        """
        _delete_activity(activity_data.get('id'))
        return {}

    def on_activity_submission_submitted(self, activity_submission_data, activity_data):
        """
        Invoked when an activity submission is submitted.
        You may override this method to react on an activity submission submitted.
        :param activity_submission_data: ronan activity submission that have been submitted
        :param activity_data: ronan activity that contains the submission
        :return: empty dictionary
        """
        return {}

    def on_activity_submission_unsubmitted(self, activity_submission_data, activity_data):
        """
        Invoked when an activity submission is unsubmitted.
        You may override this method to react on an activity submission unsubmitted.
        :param activity_submission_data: ronan activity submission that have been unsubmitted
        :param activity_data: ronan activity that contains the submission
        :return: empty dictionary
        """
        return {}

    def on_activities_reminder_sent(self, activities_by_remaining_days):
        """
        Invoked when an activities reminder is sent.
        You may override this method to react on an activities reminder sent.
        :param activities_by_remaining_days: ronan activities that have reminder configured for current day,
               grouped by remaining days
        :return: empty dictionary
        """
        return {}

    def on_internal_server_error(self, error, message_id, data, attributes, published_time):
        """
        Invoked when internal server error occur.
        You may want to log this error.
        """
        pass

    def on_value_error(self, error):
        """
        Invoked when value error occur.
        You may want to log this error.
        """
        pass



from __future__ import unicode_literals

from django.db import models
import sys
if sys.version_info[0] < 3:
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import gettext_lazy as _


class Process(models.Model):
    """
    Record process from Activity Service.
    """
    process_key = models.CharField(
        _('Process Key'), max_length=128, unique=True, db_index=True)
    process_type_slug = models.CharField(
        _('Process Type'), max_length=128)
    internal_key = models.CharField(
        _('Internal Key'), max_length=128, db_index=True)
    status = models.CharField(_('Status'), max_length=128)

    class Meta:
        app_label = 'activity'
        verbose_name = _('Process')
        verbose_name_plural = _('Processes')

    def __str__(self):
        return '%s - %s' % (self.process_type_slug, self.internal_key)


class Activity(models.Model):
    """
    Record activity from Activity Service.
    """
    activity_id = models.PositiveIntegerField(
        _('Activity ID'), unique=True, db_index=True)
    process = models.ForeignKey(
        Process,
        verbose_name=_('Process'),
        related_name='activities',
        on_delete=models.CASCADE,
    )
    from_template_activity_slug = models.CharField(
        _('Template Activity Slug'), max_length=128, null=True, default=None)

    class Meta:
        app_label = 'activity'
        verbose_name = _('Activity')
        verbose_name_plural = _('Activities')

    def __str__(self):
        return '%s - %s' % (str(self.activity_id), str(self.from_template_activity_slug))



from builtins import str

from p1_queue.base_consumer import BaseConsumer

from activity.constants import K_TRIGGER_PROCESS_ON_CREATE
from activity.constants import K_TRIGGER_PROCESS_ON_DELETE
from activity.constants import K_TRIGGER_PROCESS_ON_UPDATE
from activity.constants import K_TRIGGER_ACTIVITY_ON_COMPLETE
from activity.constants import K_TRIGGER_ACTIVITY_ON_CREATE
from activity.constants import K_TRIGGER_ACTIVITY_ON_DELETE
from activity.constants import K_TRIGGER_ACTIVITY_ON_UPDATE
from activity.constants import K_TRIGGER_ACTIVITY_SUBMISSION_ON_UNSUBMIT
from activity.constants import K_TRIGGER_ACTIVITY_SUBMISSION_ON_SUBMIT
from activity.constants import K_TRIGGER_ACTIVITIES_REMINDER_ON_SEND
from activity.services import BaseConsumerService


class ProjectActivityBaseConsumer(BaseConsumer):
    TOPIC_ID = 'INTERIOR_PROJECT'

    consumer_service = None

    def get_consumer_service(self):
        if self.consumer_service is None:
            raise NotImplementedError('No webhook service')
        if not issubclass(type(self.consumer_service), BaseConsumerService):
            raise ValueError(
                'Webhook service not inherit from BaseWebhookService')
        return self.consumer_service

    def perform_on_process_created(self, data):
        return self.get_consumer_service() \
            .on_process_created(data['process'])

    def perform_on_process_updated(self, data):
        return self.get_consumer_service() \
            .on_process_updated(data['process'])

    def perform_on_process_deleted(self, data):
        return self.get_consumer_service() \
            .on_process_deleted(data['process'])

    def perform_on_activity_completed(self, data):
        return self.get_consumer_service() \
            .on_activity_completed(data['activity'])

    def perform_on_activity_created(self, data):
        return self.get_consumer_service() \
            .on_activity_created(data['activity'])

    def perform_on_activity_updated(self, data):
        return self.get_consumer_service() \
            .on_activity_updated(data['activity'])

    def perform_on_activity_deleted(self, data):
        return self.get_consumer_service() \
            .on_activity_deleted(data['activity'])

    def perform_on_submission_submitted(self, data):
        return self.get_consumer_service() \
            .on_activity_submission_submitted(
            data['activity_submission'], data['activity'])

    def perform_on_submission_unsubmitted(self, data):
        return self.get_consumer_service() \
            .on_activity_submission_unsubmitted(
            data['activity_submission'], data['activity'])

    def perform_on_activities_reminder_sent(self, data):
        return self.get_consumer_service() \
            .on_activities_reminder_sent(
            data['activities_by_remaining_days'])

    def perform_on_value_error(self, error):
        return self.get_consumer_service() \
            .on_value_error(error)

    def perform_on_internal_server_error(self, error, message_id, data, attributes, published_time):
        return self.get_consumer_service() \
            .on_internal_server_error(error, message_id, data, attributes, published_time)

    def validate_trigger_type(self, attributes):
        if attributes['trigger_type'] == '':
            raise ValueError({'trigger_type': 'missing trigger_type'})
        return attributes['trigger_type']

    def on_message(self, message_id, data, attributes, published_time):
        try:
            trigger_type = self.validate_trigger_type(attributes)

            if trigger_type == K_TRIGGER_PROCESS_ON_CREATE:
                self.perform_on_process_created(data)
            elif trigger_type == K_TRIGGER_PROCESS_ON_UPDATE:
                self.perform_on_process_updated(data)
            elif trigger_type == K_TRIGGER_PROCESS_ON_DELETE:
                self.perform_on_process_deleted(data)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_COMPLETE:
                self.perform_on_activity_completed(data)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_CREATE:
                self.perform_on_activity_created(data)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_UPDATE:
                self.perform_on_activity_updated(data)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_DELETE:
                self.perform_on_activity_deleted(data)
            elif trigger_type == K_TRIGGER_ACTIVITY_SUBMISSION_ON_SUBMIT:
                self.perform_on_submission_submitted(data)
            elif trigger_type == K_TRIGGER_ACTIVITY_SUBMISSION_ON_UNSUBMIT:
                self.perform_on_submission_unsubmitted(data)
            elif trigger_type == K_TRIGGER_ACTIVITIES_REMINDER_ON_SEND:
                self.perform_on_activities_reminder_sent(data)
            else:
                raise ValueError({'trigger_type': 'invalid trigger_type'})
        except ValueError as e:
            self.perform_on_value_error(e)
        except Exception as e:
            self.perform_on_internal_server_error(
                e, message_id, data, attributes, published_time)



import factory

from .models import Activity, Process


class ProcessFactory(factory.DjangoModelFactory):
    process_key = factory.Faker("uuid4")

    class Meta:
        model = Process


class ActivityFactory(factory.DjangoModelFactory):
    activity_id = factory.Sequence(lambda x: "42%d" % x)
    process = factory.SubFactory(ProcessFactory)

    class Meta:
        model = Activity



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2019-07-18 14:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('activity_id', models.PositiveIntegerField(db_index=True, unique=True, verbose_name='Activity ID')),
                ('from_template_activity_slug', models.CharField(max_length=128,
                                                                 verbose_name='Template Activity Slug',
                                                                 null=True, default=None)),
            ],
            options={
                'verbose_name': 'Activity',
                'verbose_name_plural': 'Activities',
            },
        ),
        migrations.CreateModel(
            name='Process',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('process_key', models.CharField(db_index=True,
                                                 max_length=128, unique=True, verbose_name='Process Key')),
                ('internal_key', models.CharField(db_index=True,
                                                  max_length=128, verbose_name='Internal Key')),
                ('process_type_slug', models.CharField(max_length=128, verbose_name='Process Type')),
                ('status', models.CharField(max_length=128, verbose_name='Status')),
            ],
            options={
                'verbose_name': 'Process',
                'verbose_name_plural': 'Processes',
            },
        ),
        migrations.AddField(
            model_name='activity',
            name='process',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='activities', to='activity.Process', verbose_name='Process'),
        ),
    ]
